//	expdump.c
//
//	Dump exported routines of a Windows DLL
//
//	Tabs:		1,5
//
//	Compiler:	Microsoft C 8 (MSVC++)
//
//	Compile:	cl -AS -WX -Gs	expdump
//
//	Sabo								04/18/1994
//	Dedicated to Cynthia
//*****************************************************

#include <stdio.h>
#include <string.h>

//
//	defines
//
#define DEF_EXTENSION		".DLL"
#define ERRCHKMSG(c,m,l)	do{if(c){printf("  Error: %s...\n",m);goto l;}}while(0);
#define ERRCHK(c,l) 		do{if(c){goto l;}}while(0);
#define FASTCALL			__fastcall

#define FALSE				(0)
#define TRUE				(!FALSE)


//
//	my types
//
typedef struct _NEWEXEHEADER
{
		char			achSignature[2];
		unsigned char	bLnkMajor;
		unsigned char	bLnkMinor;
		unsigned int	usEntryTabOff;
		unsigned int	usEntryTabLen;
		unsigned long	ulCRC;

		unsigned int	bDGROUPType:2;
		unsigned int	fInstanceInit:1;
		unsigned int	fProtOnly:1;
		unsigned int	f8086Only:1;
		unsigned int	f80286:1;
		unsigned int	f80386:1;
		unsigned int	fFPInstructions:1;
		unsigned int	bAppType:3;
		unsigned int	fFamily:1;
		unsigned int	fReserved1:1;
		unsigned int	fErrors:1;
		unsigned int	fReserved2:1;
		unsigned int	fDLL:1;

		unsigned int	usDGROUP;
		unsigned int	usHeapSize;
		unsigned int	usStackSize;
		unsigned int	usRegIP;
		unsigned int	usSegCS;
		unsigned int	usRegSP;
		unsigned int	usSegSS;
		unsigned int	usNumSegments;
		unsigned int	usNumModRefs;
		unsigned int	usNonResidentTabLen;
		unsigned int	usSegmentTabOff;
		unsigned int	usResourceTabOff;
		unsigned int	usResidentTabOff;
		unsigned int	usModRefOff;
		unsigned int	usImportedTabOff;
		unsigned long	ulNonResidentTabOff;
		unsigned int	usNumMovables;
		unsigned int	usUnitSize;
		unsigned int	usNumResources;
		unsigned char	bTargetOS;
		unsigned char	abReserved[9];
} NEWEXEHEADER, *PNEWEXEHEADER;



//
//	static data
//
char			abTempBuf[TMP_MAX];
NEWEXEHEADER	neh;

//
// prototypes
//
int main(int argc,char **argv);
int FASTCALL DumpExports(char *pcFileName);
int FASTCALL DumpNames(FILE *pf,long lOffset,int iFirst);


int main(int argc,char **argv)
{
	int 	i;

	puts("EXPDUMP, Dump Windows DLL exports");
	puts("Written by Ralf Saborowski for CA TechniCon '94, New Orleans");
	puts("(c) Copyright Computer Associates Intl., 1993, 1994\n");


	if(argc<2)
	{
		puts("Usage: EXPDUMP <DLL-Name1> ... <DLL-Namex>");
		return(1);
	}

	for(i=1;i<argc;i++)
	{
		strcpy(abTempBuf,argv[i]);

		if(!strchr(abTempBuf,'.'))
		{
			strcat(abTempBuf,DEF_EXTENSION);
		}

		DumpExports(abTempBuf);
	}

	return(0);

}

int FASTCALL DumpExports(char *pcFileName)
{
	int 	cb;
	int 	i;
	long	l;
	FILE	*fileDLL;

	printf("Dumping exports of: %s\n",pcFileName);

	fileDLL=fopen(pcFileName,"rb");
		ERRCHKMSG(!fileDLL,"Error opening file",Error1);

	setvbuf (fileDLL, NULL, _IOFBF, 1024);

	//
	//	read beginning of DOS stub to find NE header
	//
	cb=fread(abTempBuf,1,0x40,fileDLL);
		ERRCHKMSG(cb!=0x40,"No valid Windows DLL",Error2);

	//
	//	Mark, are you there?
	//
	i=*(int*)abTempBuf;
		ERRCHKMSG(i!=0x5a4d,"Cannot find DOS stub",Error2);

	i=*(int*)(abTempBuf+0x18);
		ERRCHKMSG(i<0x40,"Sorry, this seems to be a pure DOS-EXE",Error2);

	//
	//	move to NE header
	//
	l=*(long*)(abTempBuf+0x3c);

	cb=fseek(fileDLL,l,SEEK_SET);
		ERRCHKMSG(cb,"Error moving in file",Error2);

	cb=fread(&neh,1,sizeof(NEWEXEHEADER),fileDLL);
		ERRCHKMSG(cb!=sizeof(NEWEXEHEADER),"Error reading from file",Error2);

	//
	//	check for NE signature
	//
	ERRCHKMSG(neh.achSignature[0]!='N'||neh.achSignature[1]!='E',
		"New Executeable header not found",Error2);

	//
	//	check for DLL
	//
	ERRCHKMSG(!neh.fDLL,"Not a DLL",Error2);


	//
	//	check for Windows OS
	//
	ERRCHKMSG(!neh.bTargetOS&4,"Target OS is not Windows",Error2);


	//
	//	display module name and description
	//
	cb=DumpNames(fileDLL,l+(long)neh.usResidentTabOff,1);
		ERRCHKMSG(cb,"Error processing file",Error2);
	cb=DumpNames(fileDLL,neh.ulNonResidentTabOff,2);
		ERRCHKMSG(cb,"Error processing file",Error2);


	//
	//	Dump resident names
	//
	cb=DumpNames(fileDLL,l+(long)neh.usResidentTabOff,0);
		ERRCHKMSG(cb,"Error processing file",Error2);

	//
	//	Dump non resident names
	//
	cb=DumpNames(fileDLL,neh.ulNonResidentTabOff,0);
		ERRCHKMSG(cb,"Error processing file",Error2);

	fclose(fileDLL);
	return(0);

	Error2:
		fclose(fileDLL);
	Error1:
		return(1);

}

int FASTCALL DumpNames(FILE *pf,long lOffset,int iFirst)
{
	int 	i,cb;
	int 	iOrd;

	cb=fseek(pf,lOffset,SEEK_SET);
		ERRCHK(cb,Error1);

	//
	//	first entry in res name tab is module name,
	//	in none res name tab it's the file description
	//
	cb=fread(abTempBuf,1,1,pf);
		ERRCHK(cb!=1,Error1);

	i=(int)*abTempBuf;

	cb=fread(abTempBuf,1,i+2,pf);
		ERRCHK(cb!=i+2,Error1);

	abTempBuf[i]='\0';

	if(iFirst)
	{

		switch(iFirst)
		{
			case 1:
				printf("  Modul name: %s \n",abTempBuf);
				break;

			case 2:
				printf("  Description: %s \n",abTempBuf);
		}

		return(0);
	}

	do
	{
		cb=fread(abTempBuf,1,1,pf);
			ERRCHK(cb!=1,Error1);

		i=(int)*abTempBuf;

		cb=fread(abTempBuf,1,i+2,pf);
			ERRCHK(cb!=i+2,Error1);

		iOrd=*(int*)(abTempBuf+i);

		if(i)
		{
			abTempBuf[i]='\0';

			printf("  %-40s %d\n",abTempBuf,iOrd);
		}

	} while(i);

	return(0);

	Error1:
		return(cb);

}